@@ -7,14 +7,15 @@ BEGIN {
     unless DBIx::Class::Optional::Dependencies->req_ok_for ('admin');
 }
 
+use JSON::Any qw(DWIW PP JSON CPANEL XS);
 use Moose;
 use MooseX::Types::Moose qw/Int Str Any Bool/;
 use DBIx::Class::Admin::Types qw/DBICConnectInfo DBICHashRef/;
 use MooseX::Types::JSON qw(JSON);
 use MooseX::Types::Path::Class qw(Dir File);
+use MooseX::Types::LoadableClass qw(LoadableClass);
 use Try::Tiny;
-use JSON::Any qw(DWIW XS JSON);
-use namespace::autoclean;
+use namespace::clean;
 
 =head1 NAME
 
@@ -68,7 +69,7 @@ the class of the schema to load
 
 has 'schema_class' => (
   is  => 'ro',
-  isa => Str,
+  isa => LoadableClass,
 );
 
 
@@ -87,8 +88,6 @@ has 'schema' => (
 sub _build_schema {
   my ($self)  = @_;
 
-  require Class::MOP;
-  Class::MOP::load_class($self->schema_class);
   $self->connect_info->[3]{ignore_version} = 1;
   return $self->schema_class->connect(@{$self->connect_info});
 }
@@ -167,7 +166,7 @@ sub _build_connect_info {
 
 config_file provide a config_file to read connect_info from, if this is provided
 config_stanze should also be provided to locate where the connect_info is in the config
-The config file should be in a format readable by Config::General
+The config file should be in a format readable by Config::Any.
 
 =cut
 
@@ -452,7 +451,7 @@ sub insert {
   $rs ||= $self->resultset();
   $set ||= $self->set();
   my $resultset = $self->schema->resultset($rs);
-  my $obj = $resultset->create( $set );
+  my $obj = $resultset->new_result($set)->insert;
   print ''.ref($resultset).' ID: '.join(',',$obj->id())."\n" if (!$self->quiet);
 }
 
@@ -579,6 +578,7 @@ sub _find_stanza {
       die ("Could not find $stanza in config, $path does not seem to exist.\n");
     }
   }
+  $cfg = $cfg->{connect_info} if exists $cfg->{connect_info};
   return $cfg;
 }
 
